@@ -3,7 +3,7 @@
    Parser for dhclient config and lease files... */
 
 /*
- * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: clparse.c,v 1.67.76.2 2007/02/14 22:41:22 dhankins Exp $ Copyright (c) 2004-2007 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: clparse.c,v 1.62.2.12 2006/02/22 22:43:27 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -239,7 +239,7 @@ void parse_client_statement (cfile, ip, config)
 {
 	int token;
 	const char *val;
-	struct option *option = NULL;
+	struct option *option;
 	struct executable_statement *stmt, **p;
 	enum statement_op op;
 	int lose;
@@ -288,7 +288,7 @@ void parse_client_statement (cfile, ip, config)
 		return;
 
 		/* REQUIRE can either start a policy statement or a
-		   comma-separated list of names of required options. */
+		   comma-seperated list of names of required options. */
 	      case REQUIRE:
 		next_token (&val, (unsigned *)0, cfile);
 		token = peek_token (&val, (unsigned *)0, cfile);
@@ -363,15 +363,15 @@ void parse_client_statement (cfile, ip, config)
 			return;
 		}
 
-		status = parse_option_name(cfile, 1, &known, &option);
-		if (status != ISC_R_SUCCESS || option == NULL)
+		option = parse_option_name (cfile, 1, &known);
+		if (!option)
 			return;
 
 		token = next_token (&val, (unsigned *)0, cfile);
 		if (token != CODE) {
 			parse_warn (cfile, "expecting \"code\" keyword.");
 			skip_to_semi (cfile);
-			option_dereference(&option, MDL);
+			free_option (option, MDL);
 			return;
 		}
 		if (ip) {
@@ -379,11 +379,11 @@ void parse_client_statement (cfile, ip, config)
 				    "option definitions may only appear in %s",
 				    "the outermost scope.");
 			skip_to_semi (cfile);
-			option_dereference(&option, MDL);
+			free_option (option, MDL);
 			return;
 		}
-		parse_option_code_definition(cfile, option);
-		option_dereference(&option, MDL);
+		if (!parse_option_code_definition (cfile, option))
+			free_option (option, MDL);
 		return;
 
 	      case MEDIA:
@@ -610,8 +610,7 @@ void parse_option_list (cfile, list)
 	int token;
 	const char *val;
 	pair p = (pair)0, q = (pair)0, r;
-	struct option *option = NULL;
-	isc_result_t status;
+	struct option *option;
 
 	ix = 0;
 	do {
@@ -626,8 +625,8 @@ void parse_option_list (cfile, list)
 			skip_to_semi (cfile);
 			return;
 		}
-		status = parse_option_name(cfile, 0, NULL, &option);
-		if (status != ISC_R_SUCCESS || option == NULL) {
+		option = parse_option_name (cfile, 0, NULL);
+		if (!option) {
 			parse_warn (cfile, "%s: expected option name.", val);
 			return;
 		}
@@ -636,14 +635,12 @@ void parse_option_list (cfile, list)
 				"%s.%s: Only global options allowed.",
 				option -> universe -> name, option->name );
 			skip_to_semi (cfile);
-			option_dereference(&option, MDL);
 			return;
 		}
 		r = new_pair (MDL);
 		if (!r)
 			log_fatal ("can't allocate pair for option code.");
 		r -> car = (caddr_t)(long)option -> code;
-		option_dereference(&option, MDL);
 		r -> cdr = (pair)0;
 		if (p)
 			q -> cdr = r;
@@ -1127,49 +1124,24 @@ void parse_reject_statement (cfile, config)
 {
 	int token;
 	const char *val;
-	struct iaddrmatch match;
-	struct iaddrmatchlist *list;
-	int i;
+	struct iaddr addr;
+	struct iaddrlist *list;
 
 	do {
-		if (!parse_ip_addr_with_subnet (cfile, &match)) {
-			/* no warn: parser will have reported what's wrong */
+		if (!parse_ip_addr (cfile, &addr)) {
+			parse_warn (cfile, "expecting IP address.");
 			skip_to_semi (cfile);
 			return;
 		}
 
-		/* check mask is not all zeros (because that would
-		 * reject EVERY address).  This check could be
-		 * simplified if we assume that the mask *always*
-		 * represents a prefix .. but perhaps it might be
-		 * useful to have a mask which is not a proper prefix
-		 * (perhaps for ipv6?).  The following is almost as
-		 * efficient as inspection of match.mask.iabuf[0] when
-		 * it IS a true prefix, and is more general when it is
-		 * not.
-		 */
-
-		for (i=0 ; i < match.mask.len ; i++) {
-		    if (match.mask.iabuf[i]) {
-			break;
-		    }
-		}
-
-		if (i == match.mask.len) {
-		    /* oops we found all zeros */
-		    parse_warn(cfile, "zero-length prefix is not permitted "
-				      "for reject statement");
-		    skip_to_semi(cfile);
-		    return;
-		} 
-
-		list = dmalloc(sizeof(struct iaddrmatchlist), MDL);
+		list = (struct iaddrlist *)dmalloc (sizeof (struct iaddrlist),
+						    MDL);
 		if (!list)
 			log_fatal ("no memory for reject list!");
 
-		list->match = match;
-		list->next = config->reject_list;
-		config->reject_list = list;
+		list -> addr = addr;
+		list -> next = config -> reject_list;
+		config -> reject_list = list;
 
 		token = next_token (&val, (unsigned *)0, cfile);
 	} while (token == COMMA);
