@@ -33,7 +33,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: dns.c,v 1.40 2006/07/19 17:14:55 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: dns.c,v 1.35.2.21 2006/02/22 22:43:27 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -192,7 +192,7 @@ isc_result_t enter_dns_zone (struct dns_zone *zone)
 			dns_zone_dereference (&tz, MDL);
 		}
 	} else {
-		if (!dns_zone_new_hash(&dns_zone_hash, DNS_HASH_SIZE, MDL))
+		if (!dns_zone_new_hash (&dns_zone_hash, 1, MDL))
 			return ISC_R_NOMEMORY;
 	}
 	dns_zone_hash_add (dns_zone_hash, zone -> name, 0, zone, MDL);
@@ -518,10 +518,10 @@ int get_dhcid (struct data_string *id,
 /* Now for the DDNS update code that is shared between client and
    server... */
 
-isc_result_t
-ddns_update_a(struct data_string *ddns_fwd_name, struct iaddr ddns_addr,
-	      struct data_string *ddns_dhcid, unsigned long ttl,
-	      unsigned rrsetp, unsigned conflict)
+isc_result_t ddns_update_a (struct data_string *ddns_fwd_name,
+			    struct iaddr ddns_addr,
+			    struct data_string *ddns_dhcid,
+			    unsigned long ttl, int rrsetp)
 {
 	ns_updque updqueue;
 	ns_updrec *updrec;
@@ -654,45 +654,23 @@ ddns_update_a(struct data_string *ddns_fwd_name, struct iaddr ddns_addr,
 		minires_freeupdrec (updrec);
 	}
 
-	/* If we're doing conflict resolution, we use a set of prereqs.  If
-	 * not, we delete the DHCID in addition to all A rrsets.
+	/*
+	 * DHCID RR exists, and matches client identity.
 	 */
-	if (conflict) {
-		/*
-		 * DHCID RR exists, and matches client identity.
-		 */
-		updrec = minires_mkupdrec (S_PREREQ,
-					   (const char *)ddns_fwd_name -> data,
-					   C_IN, T_DHCID, 0);
-		if (!updrec) {
-			result = ISC_R_NOMEMORY;
-			goto error;
-		}
-	
-		updrec -> r_data = ddns_dhcid -> data;
-		updrec -> r_size = ddns_dhcid -> len;
-		updrec -> r_opcode = YXRRSET;
-
-		ISC_LIST_APPEND (updqueue, updrec, r_link);
-	} else {
-		/*
-		 * Conflict detection override: delete DHCID RRs.
-		 */
-		updrec = minires_mkupdrec(S_UPDATE, ddns_fwd_name->data,
-					  C_IN, T_DHCID, 0);
-
-		if (!updrec) {
-			result = ISC_R_NOMEMORY;
-			goto error;
-		}
-
-		updrec->r_data = NULL;
-		updrec->r_size = 0;
-		updrec->r_opcode = DELETE;
-
-		ISC_LIST_APPEND(updqueue, updrec, r_link);
+	updrec = minires_mkupdrec (S_PREREQ,
+				   (const char *)ddns_fwd_name -> data,
+				   C_IN, T_DHCID, 0);
+	if (!updrec) {
+		result = ISC_R_NOMEMORY;
+		goto error;
 	}
 
+	updrec -> r_data = ddns_dhcid -> data;
+	updrec -> r_size = ddns_dhcid -> len;
+	updrec -> r_opcode = YXRRSET;
+
+	ISC_LIST_APPEND (updqueue, updrec, r_link);
+
 
 	/*
 	 * Delete A RRset.
@@ -985,4 +963,4 @@ isc_result_t ddns_remove_a (struct data_string *ddns_fwd_name,
 #endif /* NSUPDATE */
 
 HASH_FUNCTIONS (dns_zone, const char *, struct dns_zone, dns_zone_hash_t,
-		dns_zone_reference, dns_zone_dereference, do_case_hash)
+		dns_zone_reference, dns_zone_dereference)
