@@ -5,7 +5,7 @@
    transactions... */
 
 /*
- * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 2001-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char ocopyright[] =
-"$Id: trace.c,v 1.12.116.2 2007/05/23 23:30:33 each Exp $ Copyright 2004-2006 Internet Systems Consortium.";
+"$Id: trace.c,v 1.9.2.8 2006/02/22 22:43:27 dhankins Exp $ Copyright 2004-2006 Internet Systems Consortium.";
 #endif
 
 #include <omapip/omapip_p.h>
@@ -57,8 +57,6 @@ trace_type_t trace_time_marker;
 extern omapi_array_t *trace_listeners;
 extern omapi_array_t *omapi_connections;
 
-extern int errno;
-
 void trace_free_all ()
 {
 	trace_type_t *tp;
@@ -144,11 +142,6 @@ isc_result_t trace_begin (const char *filename,
 	}
 
 	traceoutfile = open (filename, O_CREAT | O_WRONLY | O_EXCL, 0600);
-	if (traceoutfile < 0 && errno == EEXIST) {
-		log_error ("WARNING: Overwriting trace file \"%s\"", filename);
-		traceoutfile = open (filename, O_WRONLY | O_EXCL, 0600);
-	}
-
 	if (traceoutfile < 0) {
 		log_error ("%s(%d): trace_begin: %s: %m",
 			   file, line, filename);
