@@ -3,7 +3,7 @@
    Parser for dhcpd config file... */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -34,7 +34,7 @@
 
 #ifndef lint
 static char copyright[] =
-"$Id: confpars.c,v 1.143.2.21 2004/06/10 17:59:51 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: confpars.c,v 1.143.2.30 2006/02/22 22:43:27 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -154,6 +154,7 @@ isc_result_t read_conf_file (const char *filename, struct group *group,
 	if (result != ulen)
 		log_fatal ("%s: short read of %d bytes instead of %d.",
 			   filename, ulen, result);
+	close (file);
       memfile:
 	/* If we're recording, write out the filename and file contents. */
 	if (trace_record ())
@@ -170,7 +171,6 @@ isc_result_t read_conf_file (const char *filename, struct group *group,
 #if defined (TRACING)
 	dfree (dbuf, MDL);
 #endif
-	close (file);
 	return status;
 }
 
@@ -1368,6 +1368,10 @@ void parse_pool_statement (cfile, group, type)
 				}
 				break;
 				
+			      case KNOWN_CLIENTS:
+				permit -> type = permit_known_clients;
+				break;
+
 			      case UNKNOWN_CLIENTS:
 				permit -> type = permit_unknown_clients;
 				break;
@@ -1460,39 +1464,53 @@ void parse_pool_statement (cfile, group, type)
 	   because BOOTP doesn't support leases, and failover absolutely
 	   depends on lease timing. */
 	if (pool -> failover_peer) {
-		for (permit = pool -> permit_list;
-		     permit; permit = permit -> next) {
+		/* This search order matches the search orders later in
+		 * execution - deny first, if not denied, check permit
+		 * list.  A dynamic bootp client may be known or unknown,
+		 * it may belong to a member of a class, but it definitely
+		 * will not be authenticated since that requires DHCP
+		 * to work.  So a dynamic bootp client is definitely not
+		 * an authenticated client, and we can't say for sure about
+		 * anything else.
+		 *
+		 * So we nag the user.
+		 */
+		for (permit = pool -> prohibit_list; permit;
+		     permit = permit -> next) {
 			if (permit -> type == permit_dynamic_bootp_clients ||
-			    permit -> type == permit_all_clients) {
-				  dynamic_bootp_clash:
-				parse_warn (cfile,
-					    "pools with failover peers %s",
-					    "may not permit dynamic bootp.");
-				log_error ("Either write a \"no failover\" %s",
-					   "statement and use disjoint");
-				log_error ("pools, or don't permit dynamic%s",
-					   " bootp.");
-				log_error ("This is a protocol limitation,%s",
-					   " not an ISC DHCP limitation, so");
-				log_error ("please don't request an %s",
-					   "enhancement or ask why this is.");
-				goto clash_testing_done;
-			}
+			    permit -> type == permit_unauthenticated_clients ||
+			    permit -> type == permit_all_clients)
+				break;
 		}
-		if (!pool -> permit_list) {
-			if (!pool -> prohibit_list)
-				goto dynamic_bootp_clash;
+		if (!permit) {
+			permit = pool -> permit_list;
+			do {
+				if (!permit ||
+				    permit -> type !=
+					permit_authenticated_clients) {
+					parse_warn (cfile,
+					  "pools with failover peers %s",
+					  "may not permit dynamic bootp.");
+					log_error ("Either write a \"%s\" %s",
+					  "no failover",
+					  "statement and use disjoint");
+					log_error ("pools, or%s (%s) %s",
+					  " don't permit dynamic bootp",
+					  "\"deny dynamic bootp clients;\"",
+					  "in this pool.");
+					log_error ("This is a protocol,%s %s",
+					   " limitation, not an ISC DHCP",
+					   "limitation, so");
+					log_error ("please don't request an %s",
+					   "enhancement or ask why this is.");
 
-			for (permit = pool -> prohibit_list; permit;
-			     permit = permit -> next) {
-				if (permit -> type ==
-				    permit_dynamic_bootp_clients ||
-				    permit -> type == permit_all_clients)
-					goto clash_testing_done;
-			}
+					break;
+				}
+
+				permit = permit -> next;
+			} while (permit);
 		}
 	}
-      clash_testing_done:				
 #endif /* FAILOVER_PROTOCOL */
 
 	/* See if there's already a pool into which we can merge this one. */
@@ -1804,7 +1822,7 @@ int parse_class_declaration (cp, cfile, group, type)
 	struct executable_statement *stmt = (struct executable_statement *)0;
 	struct expression *expr;
 	int new = 1;
-	isc_result_t status;
+	isc_result_t status = ISC_R_FAILURE;
 
 	token = next_token (&val, (unsigned *)0, cfile);
 	if (token != STRING) {
@@ -2555,6 +2573,7 @@ int parse_lease_declaration (struct lease **lp, struct parse *cfile)
 		      case TIMESTAMP:
 		      case TSTP:
 		      case TSFP:
+		      case ATSFP:
 		      case CLTT:
 			t = parse_date (cfile);
 			switch (token) {
@@ -2582,6 +2601,11 @@ int parse_lease_declaration (struct lease **lp, struct parse *cfile)
 				seenbit = 131072;
 				lease -> tsfp = t;
 				break;
+
+			      case ATSFP:
+				seenbit = 262144;
+				lease->atsfp = t;
+				break;
 				
 			      case CLTT:
 				seenbit = 524288;
@@ -2589,7 +2613,8 @@ int parse_lease_declaration (struct lease **lp, struct parse *cfile)
 				break;
 				
 			      default: /* for gcc, we'll never get here. */
-				break;
+				log_fatal ("Impossible error at %s:%d.", MDL);
+				return 0;
 			}
 			break;
 
@@ -2730,7 +2755,7 @@ int parse_lease_declaration (struct lease **lp, struct parse *cfile)
 					    "%s: expecting a binding state.",
 					    val);
 				skip_to_semi (cfile);
-				break;
+				return 0;
 			}
 
 			if (seenbit == 256) {
@@ -2904,11 +2929,13 @@ int parse_lease_declaration (struct lease **lp, struct parse *cfile)
 					       "name");
 			    strcpy (binding -> name, val);
 			    newbinding = 1;
-			} else if (binding -> value) {
-				binding_value_dereference (&binding -> value,
+			} else  {
+				if (binding -> value)
+				  binding_value_dereference (&binding -> value,
 							   MDL);
 				newbinding = 0;
 			}
+
 			if (!binding_value_allocate (&binding -> value, MDL))
 				log_fatal ("no memory for binding value.");
 
